/**
 * @file
 * JS for OnPoint predictive search.
 */
// modified from: https://github.com/mocanuga/inlinePredictor
(function (drupalSettings) {
  "use strict";

  /* HELPER FUNCTIONS */
  /**
   * Get all siblings of an element
   * (c) 2021 Chris Ferdinandi, MIT License, https://gomakethings.com
   * @param  {Node}  elem The element
   * @return {Array}      The siblings
   */
  const getSiblings = function (elem, selector) {
    return Array.from(elem.parentNode.children).filter(function (sibling) {
      return sibling !== elem && sibling.matches(selector);
    });
  };

  /**
   * https://gist.github.com/gf3/132080/110d1b68d7328d7bfe7e36617f7df85679a08968
   */
  const loadJSONP = (function () {
    let unique = 0;

    return function (url, callback, context) {
      // INIT
      const name = "_jsonp_" + unique++;

      if (url.match(/\?/)) {
        url += "&callback=" + name;
      } else {
        url += "?callback=" + name;
      }

      // Create script
      let script = document.createElement("script");
      script.type = "text/javascript";
      script.src = url;

      // Setup handler
      window[name] = function (data) {
        callback.call(context || window, data);
        document.getElementsByTagName("head")[0].removeChild(script);
        script = null;
        delete window[name];
      };

      // Load JSON
      document.getElementsByTagName("head")[0].appendChild(script);
    };
  })();

  const debounce = function (delay, fn) {
    let inDebounce = null;

    return (args) => {
      clearTimeout(inDebounce);
      inDebounce = setTimeout(() => fn(args), delay);
    };
  };
  /* END HELPER FUNCTIONS */

  const predict = function () {
    const defaults = {
      sentenceMode: false,
      stopRule: false,
      service: "",
      prediction: null,
    };
    let elems = Object.create(null);
    let opts = Object.create(null);
    let active = null;
    let last;
    let predictions;

    function extend(destination, source) {
      let property;

      for (property in source) {
        if (
          source[property] &&
          source[property].constructor &&
          source[property].constructor === Object
        ) {
          destination[property] = destination[property] || {};
          extend(destination[property], source[property]);
        } else {
          destination[property] = source[property];
        }
      }

      return destination;
    }

    function select(el, start, end) {
      if (el.setSelectionRange) {
        el.setSelectionRange(start, end);
      } else if (el.createTextRange) {
        const sel = el.createTextRange();

        sel.collapse(true);
        sel.moveStart("character", start);
        sel.moveEnd("character", end);
        sel.select();
      } else if (typeof el.selectionStart != "undefined") {
        el.selectionStart = start;
        el.selectionEnd = end;
      }

      el.focus();
    }

    function handle(t, p, w, v) {
      let l = v.length;
      const s = (function (v, l) {
        if (!l || w.lastIndexOf(v, 0) !== 0) {
          for (let i = 0, j = p.length; i < j; i++) {
            if (p[i].toLowerCase().lastIndexOf(v, 0) === 0) {
              last = p[i].toLowerCase();

              break;
            }
          }
        }

        return !!last ? last.toLowerCase().replace(v, "") : "";
      })(v, last);
      const i = opts.sentenceMode ? w.join(" ") : w;

      t.value = i + s;
      l = t.value.length;
      select(t, l - s.length, l);

      if (typeof opts.prediction === "function") {
        opts.prediction.call(null, t, t.value, i, s);
      }
    }

    function defer(t, w, v = "") {
      const processPredict = function (data) {
        if (data.suggestions.length && Array.isArray(data.suggestions)) {
          data.suggestions.sort((a, b) =>
            a.weight > b.weight
              ? 1
              : a.weight === b.weight
              ? a.term > b.term
                ? 1
                : -1
              : -1
          );

          const r = data.suggestions.map((x) => x.term);

          predictions = r.slice();
          handle(t, r, w, v);
        }
      };

      // terms lookup URI
      let termsUrl;

      // @TODO: remove 'encodeURI("(*)"))' on next OPS update
      if (
        "fr" === drupalSettings.onPointSearch.language &&
        drupalSettings.onPointSearch.onPointFr
      ) {
        termsUrl =
          drupalSettings.onPointSearch.onPointApiUrl +
          "/api/search-suggest?search_engine=" +
          drupalSettings.onPointSearch.onPointKey +
          "&url_filter=" +
          (drupalSettings.onPointSearch.onPointUrlFilterFr
            ? encodeURI(drupalSettings.onPointSearch.onPointUrlFilterFr)
            : encodeURI("(*)")) +
          "&url_exclude_filter=" +
          (drupalSettings.onPointSearch.onPointUrlExcludeFilterFr
            ? encodeURI(drupalSettings.onPointSearch.onPointUrlExcludeFilterFr)
            : "") +
          "&q=" +
          v;
      } else {
        termsUrl =
          drupalSettings.onPointSearch.onPointApiUrl +
          "/api/search-suggest?search_engine=" +
          drupalSettings.onPointSearch.onPointKey +
          "&url_filter=" +
          (drupalSettings.onPointSearch.onPointUrlFilter
            ? encodeURI(drupalSettings.onPointSearch.onPointUrlFilter)
            : encodeURI("(*)")) +
          "&url_exclude_filter=" +
          (drupalSettings.onPointSearch.onPointUrlExcludeFilter
            ? encodeURI(drupalSettings.onPointSearch.onPointUrlExcludeFilter)
            : "") +
          "&q=" +
          v;
      }

      loadJSONP(encodeURI(termsUrl), processPredict);
    }

    const kd = debounce(100, function (e) {
      const t = e.target;
      const start = t.selectionStart;
      const tv = t.value;
      const v = tv.substr(0, start);

      // get current suggestions list
      const currentSuggestions = getSiblings(t, ".search-suggestions")[0];

      // backspace/delete keys
      if (e.keyCode == 8 || e.keyCode == 46) {
        e.target.value = v;

        // hide suggestions list from view
        if (currentSuggestions) {
          currentSuggestions.classList.remove("active");
        }

        return;
      }

      if (!!opts.stopRule && !!v.match(opts.stopRule)) {
        e.preventDefault();
        e.stopPropagation();
      }
    });

    var kp = debounce(250, function (e) {
      // skip modifier keys
      if (
        e.which !== 0 &&
        e.keyCode !== 0 &&
        (e.ctrlKey || e.metaKey || e.altKey)
      ) {
        return;
      }

      e.stopPropagation();
      e.preventDefault();

      const t = e.target;
      const tv = t.value;
      let v;
      let l;

      // get current suggestions list
      const currentSuggestions = getSiblings(t, ".search-suggestions")[0];

      // hide suggestions list from view
      if (currentSuggestions) {
        currentSuggestions.classList.remove("active");
      }

      if (opts.sentenceMode) {
        const w = tv.split(" ");
        v = w.slice(-1)[0].toLowerCase();
      } else {
        v = tv;
      }

      l = v.length;

      if (!l) {
        return true;
      }

      if (!!opts.stopRule && !!v.match(opts.stopRule)) {
        t.value = tv;

        return false;
      }

      if (!last || (!!last && last.lastIndexOf(v, 0) !== 0)) {
        defer(t, opts.sentenceMode ? w : tv, v);
        last = null;
      } else {
        handle(t, predictions, opts.sentenceMode ? w : tv, v);
      }
    });

    function activate(e) {
      if (
        !!elems[e.target.id] &&
        !e.target.isSameNode(active || document.createElement("a"))
      ) {
        active = e.target;
        opts = elems[e.target.id].opts;
      }
    }

    function deactivate(e) {
      active = null;
      opts = Object.create(null);
    }

    return {
      bind: function (element, options) {
        element.addEventListener("focusin", activate, true);
        element.addEventListener("focusout", deactivate, true);
        element.addEventListener("keypress", kp, true);
        element.addEventListener("keydown", kd, true);
        elems[element.getAttribute("id")] = {};
        elems[element.getAttribute("id")].opts = extend(defaults, options);
      },
      unbind: function (element) {
        element.removeEventListener("focusin", activate, true);
        element.removeEventListener("focusout", deactivate, true);
        element.removeEventListener("keypress", kp, true);
        element.removeEventListener("keydown", kd, true);
        delete elems[element.getAttribute("id")];
      },
    };
  };

  window["predict"] = new predict();
})(drupalSettings);
