/**
 * @file
 * JS for OnPoint suggested search.
 */
(function (Drupal, drupalSettings) {
  "use strict";
  // D8 behaviors: https://www.drupal.org/docs/8/api/javascript-api/javascript-api-overview
  Drupal.behaviors.onpointSuggest = {
    attach: function (context, settings) {
      /* HELPER FUNCTIONS */
      /**
       * Get all siblings of an element
       * (c) 2021 Chris Ferdinandi, MIT License, https://gomakethings.com
       * @param  {Node}  elem The element
       * @return {Array}      The siblings
       */
      const getSiblings = function (elem, selector) {
        return Array.from(elem.parentNode.children).filter(function (sibling) {
          return sibling !== elem && sibling.matches(selector);
        });
      };

      /**
       * Convert a template string into HTML DOM nodes
       * @param  {String} str The template string
       * @return {Node}       The template HTML
       */
      function stringToHTML(str) {
        var dom = document.createElement("div");
        dom.innerHTML = str;
        return dom.firstChild;
      }
      /* END HELPER FUNCTIONS */

      // OPS suggested search functionality
      function onPointSuggest() {
        // if OPS settings configured
        if (
          !!drupalSettings.onPointSearch.onPointKey &&
          !!drupalSettings.onPointSearch.onPointInput
        ) {
          // if suggested search input and terms configured
          if (
            !!drupalSettings.onPointSearch.onPointPrimary &&
            !!drupalSettings.onPointSearch.onPointSuggested
          ) {
            // get suggested search terms, filter out empty elements
            let opSuggestions = drupalSettings.onPointSearch.onPointSuggested
              .split("\n")
              .filter((n) => n);

            // loop through suggested search terms and build list item link nodes, save to array
            opSuggestions = opSuggestions.map((suggestion) =>
              stringToHTML(
                '<li><a href="' +
                  window.location.protocol +
                  "//" +
                  window.location.host +
                  ("fr" === drupalSettings.onPointSearch.language &&
                  drupalSettings.onPointSearch.onPointFr
                    ? drupalSettings.onPointSearch.onPointPathFr
                    : drupalSettings.onPointSearch.onPointPath) +
                  "?opq=" +
                  encodeURI(Drupal.t(suggestion.trim())) +
                  '" tabindex="0">' +
                  Drupal.t(suggestion) +
                  "</a></li>"
              )
            );

            // get primary search inputs from module settings, filter for unique values
            const opPrimaryInputs = drupalSettings.onPointSearch.onPointPrimary
              .split(",")
              .filter((n) => n.trim());

            if (opPrimaryInputs.length && opSuggestions.length) {
              // loop through input setting values
              opPrimaryInputs.forEach((opInput) => {
                // get all current inputs by selector
                const currentInputs = context.querySelectorAll(opInput);

                if (currentInputs.length) {
                  // loop through each input and bind suggested search
                  currentInputs.forEach((primaryInput) => {
                    // append markup (as DOM node) for search suggestions to input
                    primaryInput.parentNode.insertBefore(
                      stringToHTML(
                        '<ul class="search-suggestions" role="listbox" tabindex="-1"><li><h6>' +
                          Drupal.t(
                            drupalSettings.onPointSearch.onPointSuggestedTitle +
                              ":"
                          ) +
                          "</h6></li></ul>"
                      ),
                      primaryInput.nextSibling
                    );

                    // get current suggestions list
                    const currentSuggestions = getSiblings(
                      primaryInput,
                      ".search-suggestions"
                    )[0];

                    // append suggestions into search markup (passes in each linked suggestion from array of DOM nodes)
                    opSuggestions.forEach(opSuggestion => currentSuggestions.appendChild(opSuggestion));

                    // show search suggestions on input focusin
                    primaryInput.addEventListener("focusin", () => {
                      if (
                        "none" ===
                        window.getComputedStyle(currentSuggestions).display
                      ) {
                        currentSuggestions.classList.add("active");
                      }
                    });

                    // hide search suggestions on input focusout
                    primaryInput.parentNode.addEventListener("focusout", () => {
                      setTimeout(() => {
                        if (
                          !primaryInput.parentNode.contains(
                            document.activeElement
                          )
                        ) {
                          currentSuggestions.classList.remove("active");
                        }
                      }, 100);
                    });

                    // hide search suggestions on focusin of any form elements after search input and search suggestions
                    primaryInput.parentNode
                      .querySelector(".search-suggestions ~ *")
                      .addEventListener("focusin", () =>
                        currentSuggestions.classList.remove("active")
                      );
                  });
                }
              });
            }
          }
        }
      }

      window.addEventListener("DOMContentLoaded", onPointSuggest());
    },
  };
})(Drupal, drupalSettings);
