<?php

namespace Drupal\onpoint_search_d8\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Component\Utility\Xss;
use Drupal\Core\Cache\Cache;

/**
 * Configure OnPoint Search settings for this site.
 */
class OnPointSettingsForm extends ConfigFormBase {
  /**
 * @var string Config settings */
  const SETTINGS = 'onpoint_search_d8.settings';

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'onpoint_admin_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    // OnPoint results page path validation.
    global $base_url;

    // Validate each search results path to ensure that the alias doesn't already exist.
    foreach (['onpoint_path', 'onpoint_path_fr'] as $search_path) {
      // Get the language code.
      $langcode = !strpos($search_path, 'fr') ? \Drupal::languageManager()->getCurrentLanguage()->getId() : 'fr';

      // Get the current search path value.
      ${$search_path} = $form_state->getValue($search_path);

      // Check if the alias exists.
      $path_exists = ${$search_path} ? \Drupal::service('path.alias_storage')->aliasExists(${$search_path}, $langcode, $base_url) : FALSE;

      // If current search path is set and alias exists, trigger error.
      if (!empty(${$search_path}) && $path_exists) {
        $form_state->setErrorByName($search_path, $this->t('This path is already in use. Please choose a unique path that is not already being used on your site.'));
      }
    }

  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      static::SETTINGS,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config(static::SETTINGS);

    // API key field.
    $form['onpoint_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API key'),
      '#default_value' => $config->get('onpoint_key'),
      '#description' => $this->t('Enter your OnPoint Search API key.'),
      '#required' => TRUE,
      '#size' => 36,
      '#maxlength' => 36,
      // Ensure valid API key format.
      '#pattern' => '^([0-9A-Fa-f]{8})-([0-9A-Fa-f]{4})-([0-9A-Fa-f]{4})-([0-9A-Fa-f]{4})-([0-9A-Fa-f]{12})$',
    ];

    // Visual hr separator element.
    $form['onpoint_hr1'] = [
      '#type' => 'item',
      '#description' => '<hr />',
    ];

    // Search input identifier, this is used to tie the identified input to OPS search and the OPS SERP.
    $form['onpoint_input'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Search input'),
      '#default_value' => $config->get('onpoint_input'),
      '#description' => $this->t('Enter the identifier for search input(s). If more than one, separate values with a comma. (E.g. \'.search-input, .search-input2\')'),
      '#maxlength' => 60,
    ];

    // Checkbox to enable OPS predictive autocomplete — conditionally includes necessary JS if enabled.
    $form['onpoint_predict'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable predictive autocomplete?'),
      '#default_value' => $config->get('onpoint_predict'),
      '#description' => $this->t('Enables predictive autcomplete for search input fields. Autocomplete suggestions are returned from the OnPoint API.'),
    ];

    // Checkbox to enable OPS search suggestions — conditionally includes necessary JS if enabled.
    $form['onpoint_suggest'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable suggested search terms?'),
      '#default_value' => $config->get('onpoint_suggest'),
      '#description' => $this->t('Enables suggested search terms for an indentified search input.'),
    ];

    // Input for search input identifer to bind search suggestions to.
    $form['onpoint_primary'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Primary input'),
      '#default_value' => $config->get('onpoint_primary'),
      '#description' => $this->t('Enter the identifier for primary search input(s) that are to show configured \'Suggested Search Terms\'. If more than one, separate values with a comma. (E.g. \'#search-1, #search-2\')'),
      '#maxlength' => 60,
      '#states' => [
        // Show this field if suggested terms are enabled.
        'visible' => [
          ':input[name="onpoint_suggest"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Input for suggested search section title (appears below search field when field is focused).
    $form['onpoint_suggested_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Suggested search title'),
      '#default_value' => $config->get('onpoint_suggested_title'),
      '#description' => $this->t('Enter the title you wish to display for suggested search terms. Default title is \'Popular Searches\'. This value may be translated through the admin \'User interface translation\'.'),
      '#states' => [
        // Show this field if suggested terms are enabled.
        'visible' => [
          ':input[name="onpoint_suggest"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Textarea input to configure suggested search terms.
    $form['onpoint_suggested'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Suggested search terms'),
      '#default_value' => $config->get('onpoint_suggested'),
      '#description' => $this->t('Enter suggested search terms you wish to display. Enter each term on a new line. These values may be translated through the admin \'User interface translation\'.'),
      '#states' => [
        // Show this field if suggested terms are enabled.
        'visible' => [
          ':input[name="onpoint_suggest"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Visual hr separator element.
    $form['onpoint_hr2'] = [
      '#type' => 'item',
      '#description' => '<hr />',
    ];

    // Input for SERP title.
    $form['onpoint_title'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Results page title'),
      '#default_value' => $config->get('onpoint_title'),
      '#description' => $this->t('Enter the page title for your OnPoint Search results page. Default title is \'Search Results\'. This value may be translated through the admin \'User interface translation\'.'),
      '#maxlength' => 60,
    ];

    // Input for SERP path.
    $form['onpoint_path'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Results page path'),
      '#default_value' => $config->get('onpoint_path'),
      '#description' => $this->t('Enter the desired path for the OnPoint Search results page. This must be a unique path that is not already being used on your site. Default path is \'/search-results\'.'),
      '#maxlength' => 60,
    ];

    // Input for search results URL filter.
    $form['onpoint_url_filter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL filter'),
      '#default_value' => $config->get('onpoint_url_filter'),
      '#description' => $this->t('Enter the regular expression to be used for the search results URL filter. The search results will be limited to <em>only</em> those matching the value entered here.<br>*Note: The value for this field must be entered in <a href="https://solr.apache.org/guide/6_6/the-standard-query-parser.html" target="_blank">SOLR’s Lucene REGEX syntax</a> (additional reference resources available <a href="https://help.oncrawl.com/en/articles/1685982-lucene-regex-cheat-sheet" target="_blank">here</a>).'),
      '#maxlength' => 250,
    ];

    // Input for search results URL exclude filter.
    $form['onpoint_url_exclude_filter'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL exclude filter'),
      '#default_value' => $config->get('onpoint_url_exclude_filter'),
      '#description' => $this->t('Enter the regular expression to be used for the search results URL exclude filter. The search results will <em>exclude</em> any results matching the value entered here.<br>*Note: The value for this field must be entered in <a href="https://solr.apache.org/guide/6_6/the-standard-query-parser.html" target="_blank">SOLR’s Lucene REGEX syntax</a> (additional reference resources available <a href="https://help.oncrawl.com/en/articles/1685982-lucene-regex-cheat-sheet" target="_blank">here</a>).'),
      '#maxlength' => 250,
    ];

    // Checkbox to enable French language support.
    $form['onpoint_fr'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable French language support?'),
      '#default_value' => $config->get('onpoint_fr'),
      '#description' => $this->t('Enables French language support and associated configuration fields.'),
    ];

    // Input for FR SERP path.
    $form['onpoint_path_fr'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Results page path (FR)'),
      '#default_value' => $config->get('onpoint_path_fr'),
      '#description' => $this->t('Enter the desired path for a unique French language search results page. This must be a unique path that is not already being used on your site. Default path is \'/fr/search-results\'.'),
      '#maxlength' => 60,
      '#states' => [
        // Show this field if French language support is enabled.
        'visible' => [
          ':input[name="onpoint_fr"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Input for FR search results URL filter.
    $form['onpoint_url_filter_fr'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL filter (FR)'),
      '#default_value' => $config->get('onpoint_url_filter_fr'),
      '#description' => $this->t('Enter the regular expression to be used for the French search results URL filter. The search results will be limited to <em>only</em> those matching the value entered here.<br>*Note: The value for this field must be entered in <a href="https://solr.apache.org/guide/6_6/the-standard-query-parser.html" target="_blank">SOLR’s Lucene REGEX syntax</a> (additional reference resources available <a href="https://help.oncrawl.com/en/articles/1685982-lucene-regex-cheat-sheet" target="_blank">here</a>).'),
      '#maxlength' => 250,
      '#states' => [
        // Show this field if French language support is enabled.
        'visible' => [
          ':input[name="onpoint_fr"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Input for FR search results URL filter.
    $form['onpoint_url_exclude_filter_fr'] = [
      '#type' => 'textfield',
      '#title' => $this->t('URL exclude filter (FR)'),
      '#default_value' => $config->get('onpoint_url_exclude_filter_fr'),
      '#description' => $this->t('Enter the regular expression to be used for the French search results URL exclude filter. The search results will <em>exclude</em> any results matching the value entered here.<br>*Note: The value for this field must be entered in <a href="https://solr.apache.org/guide/6_6/the-standard-query-parser.html" target="_blank">SOLR’s Lucene REGEX syntax</a> (additional reference resources available <a href="https://help.oncrawl.com/en/articles/1685982-lucene-regex-cheat-sheet" target="_blank">here</a>).'),
      '#maxlength' => 250,
      '#states' => [
        // Show this field if French language support is enabled.
        'visible' => [
          ':input[name="onpoint_fr"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Visual hr separator element.
    $form['onpoint_hr3'] = [
      '#type' => 'item',
      '#description' => '<hr />',
    ];

    // Textarea input for custom SERP CSS.
    $form['onpoint_css'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Custom CSS'),
      '#default_value' => $config->get('onpoint_css'),
      '#description' => $this->t('Enter any custom CSS for the OnPoint Search results page here.'),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Get SERP path.
    $onpoint_path = $form_state->getValue('onpoint_path');
    $onpoint_path = (!empty($onpoint_path) && substr($onpoint_path, 0, 1) !== '/') ? '/' . $onpoint_path : $onpoint_path;

    // Get FR SERP path.
    $onpoint_path_fr = $form_state->getValue('onpoint_path_fr');
    $onpoint_path_fr = (!empty($onpoint_path_fr) && substr($onpoint_path_fr, 0, 1) !== '/') ? '/' . $onpoint_path_fr : $onpoint_path_fr;

    // Get custom CSS.
    $onpoint_css = $form_state->getValue('onpoint_css');

    // Filter to prevent cross-site-scripting (XSS) vulnerabilities.
    $onpoint_css_min = Xss::filter($onpoint_css);

    // Perform some basic minification on the CSS.
    if (!empty($onpoint_css)) {
      // Remove comments.
      $onpoint_css_min = preg_replace('/\/\*((?!\*\/).)*\*\//', '', $onpoint_css);

      // Collapse whitespace.
      $onpoint_css_min = preg_replace('/\s{2,}/', ' ', $onpoint_css_min);

      // Remove whitespace where not needed.
      $onpoint_css_min = preg_replace('/\s*([:;{}])\s*/', '$1', $onpoint_css_min);

      // Remove final semicolon.
      $onpoint_css_min = preg_replace('/;}/', '}', $onpoint_css_min);

      // Enable GZip encoding.
      ob_start('ob_gzhandler');
    }

    // Write CSS to file.
    $css_file_uri = file_unmanaged_save_data($onpoint_css_min, 'public://onpoint_search_d8--custom-styles.min.css', FILE_EXISTS_REPLACE);

    // Get path to CSS file.
    $css_path = parse_url(file_create_url($css_file_uri))['path'];

    // Retrieve the configuration and save.
    $this->configFactory->getEditable(static::SETTINGS)
      // Set the submitted configuration settings.
      ->set('onpoint_key', $form_state->getValue('onpoint_key'))
      ->set('onpoint_input', $form_state->getValue('onpoint_input'))
      ->set('onpoint_predict', $form_state->getValue('onpoint_predict'))
      ->set('onpoint_suggest', $form_state->getValue('onpoint_suggest'))
      ->set('onpoint_fr', $form_state->getValue('onpoint_fr'))
      ->set('onpoint_primary', $form_state->getValue('onpoint_primary'))
      ->set('onpoint_suggested_title', $form_state->getValue('onpoint_suggested_title'))
      ->set('onpoint_suggested', $form_state->getValue('onpoint_suggested'))
      ->set('onpoint_title', $form_state->getValue('onpoint_title'))
      ->set('onpoint_path', $onpoint_path)
      ->set('onpoint_url_filter', $form_state->getValue('onpoint_url_filter'))
      ->set('onpoint_url_exclude_filter', $form_state->getValue('onpoint_url_exclude_filter'))
      ->set('onpoint_url_filter_fr', $form_state->getValue('onpoint_url_filter_fr'))
      ->set('onpoint_url_exclude_filter_fr', $form_state->getValue('onpoint_url_exclude_filter_fr'))
      ->set('onpoint_path_fr', $onpoint_path_fr)
      ->set('onpoint_css', $form_state->getValue('onpoint_css'))
      ->set('onpoint_css_file_path', $css_path)
      ->save();

    if ($onpoint_css !== '') {
      // Display confirmation that file has been created.
      $messenger = \Drupal::messenger();
      $messenger->addMessage(t('Custom CSS saved to the server, file URL is "%url"',
          ['%url' => file_create_url($css_file_uri)])
      );
    }

    // Invalidate the cache.
    Cache::invalidateTags(['onpoint_search_d8_lib']);

    // Submit settings.
    parent::submitForm($form, $form_state);
  }

}
